package de.jalin.ldapadmin.ldap;

import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchResult;

import de.jalin.ldapadmin.beans.Group;
import de.jalin.ldapadmin.beans.User;

public class GroupsDAO {

    private final LDAPSession session;

    public GroupsDAO(final LDAPSession session) {
        this.session = session;
    }

    public SortedMap<String, Group> loadGroups(final SortedMap<String, User> users) throws LDAPSessionException {
        final SortedMap<String, Group> list;
        list = new TreeMap<>();
        final List<SearchResult> searchResult = session.search("ou=groups");
        for (final SearchResult result : searchResult) {
            final Attributes attribs = result.getAttributes();
            final Group grp = new Group();
            grp.setName(session.getStringValue(attribs, "cn"));
            grp.setDn(result.getNameInNamespace());
            final List<String> listOfMembers = session.getListOfValues(attribs, "uniqueMember");
            final String dn = grp.getDn();
            for (final String userDN : listOfMembers) {
            	final User user = users.get(userDN);
            	if (user != null) {
            		final List<String> groups = user.getGroups();
            		groups.add(dn);
            	} else {
            		System.out.println("Error: " + userDN + " in group " + dn);
            	}
            }
            grp.setMembers(listOfMembers);
            list.put(dn, grp);
        }
        return list;
    }

    public void create(final Group grp) throws LDAPSessionException, AlreadyBoundException {
        assert grp != null;
        final String name = grp.getName();
        assert name != null;
        final BasicAttributes attributes = new BasicAttributes();
        final BasicAttribute objClass = new BasicAttribute("objectClass");
        objClass.add("top");
        objClass.add("groupOfUniqueNames");
        attributes.put(objClass);
        attributes.put("cn", name);
        final List<String> uniqueMembers = grp.getMembers();
        final BasicAttribute uniqMembers = new BasicAttribute("uniqueMember");
        if (uniqueMembers != null && uniqueMembers.size() > 0) {
            uniqueMembers.forEach((dn) -> {
                uniqMembers.add(dn);
            });
        }
        attributes.put(uniqMembers);
        final String dn = session.createSubcontext("cn=${cn},ou=groups".replace("${cn}", name), attributes);
        grp.setDn(dn);
    }

    public Group readGroup(final String dn, final SortedMap<String, User> users) throws LDAPSessionException {
        final Attributes attribs = session.getAttributes(dn.substring(0, dn.indexOf("ou=groups") + 9));
        final Group grp = new Group();
        grp.setDn(dn);
        grp.setName(session.getStringValue(attribs, "cn"));
        final List<String> listOfMembers = session.getListOfValues(attribs, "uniqueMember");
        listOfMembers.stream().map((userDN) -> users.get(userDN)).map((user) -> user.getGroups()).forEachOrdered((groups) -> {
            groups.add(dn);
        });
        grp.setMembers(listOfMembers);
        return grp;
    }

    public void update(final Group grp) throws LDAPSessionException, NoGroupMembersException {
        assert grp != null;
        final String name = grp.getName();
        assert name != null;
        if (grp.getMembers().isEmpty()) {
            throw new NoGroupMembersException(name);
        }
        final BasicAttribute membersOfAttrib = new BasicAttribute("uniqueMember");
        grp.getMembers().forEach((memberDN) -> {
            membersOfAttrib.add(memberDN);
        });
        final ModificationItem modificationItem = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, membersOfAttrib);
        session.modifyAttributes("cn=${cn},ou=groups".replace("${cn}", name), new ModificationItem[]{modificationItem});
    }

    public void updateMemberships(final User usr) throws LDAPSessionException, NoGroupMembersException {
        final List<SearchResult> searchResult = session.search("ou=groups");
        for (final SearchResult result : searchResult) {
            final Attributes attribs = result.getAttributes();
            final Group grp = new Group();
            grp.setName(session.getStringValue(attribs, "cn"));
            grp.setDn(result.getNameInNamespace());
            final List<String> listOfGroupMembers = session.getListOfValues(attribs, "uniqueMember");
            grp.setMembers(listOfGroupMembers);
            final List<String> listOfUserMemberships = usr.getGroups();
            if (listOfGroupMembers.contains(usr.getDn()) && !listOfUserMemberships.contains(grp.getDn())) {
                grp.getMembers().remove(usr.getDn());
                update(grp);
            }
            if (!listOfGroupMembers.contains(usr.getDn()) && listOfUserMemberships.contains(grp.getDn())) {
                grp.getMembers().add(usr.getDn());
                update(grp);
            }
        }
    }

    public void delete(final Group grp) throws LDAPSessionException {
        assert grp != null;
        session.unbind("cn=${id},ou=groups", grp.getName());
    }

}
