package de.jalin.ldapadmin.ldap;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.NameAlreadyBoundException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.NoPermissionException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchResult;

public class LDAPSession {

    private InitialDirContext ctx;
    private String connectString;

    public LDAPSession(final String providerURL, final String principal, final String password) throws LDAPSessionException {
        final Properties env = new Properties();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put("com.sun.jndi.ldap.connect.pool", "true");
        env.put(Context.PROVIDER_URL, providerURL);
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_PRINCIPAL, principal);
        env.put(Context.SECURITY_CREDENTIALS, password);
        connectString = providerURL;
        try {
            ctx = new InitialDirContext(env);
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
        createOrgUnitNodesIfNotExist();
    }

    public String getStringValue(final Attributes attribs, final String attrName) throws LDAPSessionException {
        final Attribute attribute = attribs.get(attrName);
        if (attribute == null) {
            return null;
        }
        try {
            return (String) attribute.get();
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public byte[] getBytesValue(final Attributes attribs, final String attrName) throws LDAPSessionException {
        final Attribute attribute = attribs.get(attrName);
        if (attribute == null) {
            return null;
        }
        try {
            return (byte[]) attribute.get();
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public List<String> getListOfValues(final Attributes attribs, final String attrName) throws LDAPSessionException {
        final Attribute attribute = attribs.get(attrName);
        if (attribute == null) {
            return null;
        }
        try {
            final int size = attribute.size();
            List<String> listOfValues;
            listOfValues = new ArrayList<>();
            for (int idx = 0; idx < size; idx++) {
                listOfValues.add((String) attribute.get(idx));
            }
            return listOfValues;
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public String createSubcontext(final String subcontext, final BasicAttributes attributes) throws LDAPSessionException, AlreadyBoundException {
        try {
            final DirContext dirContext = ctx.createSubcontext(subcontext, attributes);
            return dirContext.getNameInNamespace();
        } catch (NameAlreadyBoundException e) {
            throw new AlreadyBoundException(subcontext);
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public List<SearchResult> search(final String name, final String attribName, final String attribValue) throws LDAPSessionException {
        final List<SearchResult> searchResult;
        searchResult = new ArrayList<>();
        try {
            Attributes matchingAttributes = new BasicAttributes();
            matchingAttributes.put(attribName, attribValue);
            final NamingEnumeration<SearchResult> searchEnum = ctx.search(name, matchingAttributes);
            while (searchEnum.hasMore()) {
                searchResult.add(searchEnum.next());
            }
            return searchResult;
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public List<SearchResult> search(final String name) throws LDAPSessionException {
        final List<SearchResult> searchResult;
        searchResult = new ArrayList<>();
        try {
            final NamingEnumeration<SearchResult> searchEnum = ctx.search(name, null);
            while (searchEnum.hasMore()) {
                searchResult.add(searchEnum.next());
            }
            return searchResult;
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public Attributes getAttributes(final String dn) throws LDAPSessionException {
        try {
            return ctx.getAttributes(dn);
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public void modifyAttributes(final String name, final ModificationItem[] mods) throws LDAPSessionException {
        try {
            ctx.modifyAttributes(name, mods);
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }

    }

    public void unbind(final String searchPattern, final String id) throws LDAPSessionException {
        try {
            ctx.unbind(searchPattern.replace("${id}", id));
        } catch (NamingException e) {
            throw new LDAPSessionException(e);
        }
    }

    public void close() throws NamingException {
        if (ctx != null) {
            ctx.close();
            ctx = null;
        }
    }
    
    public String getSessionInfo() {
    	return connectString;
    }

    private void createOrgUnitNodesIfNotExist() throws LDAPSessionException {
        try {
            final BasicAttributes usersAttributes = new BasicAttributes();
            final BasicAttribute usersObjClass = new BasicAttribute("objectClass");
            usersObjClass.add("top");
            usersObjClass.add("organizationalUnit");
            usersAttributes.put(usersObjClass);
            usersAttributes.put("ou", "users");
            ctx.createSubcontext("ou=users", usersAttributes);
        } catch (NamingException e) {
            if (!((e instanceof NameAlreadyBoundException) || (e instanceof NoPermissionException))) {
                throw new LDAPSessionException(e);
            }
        }
        try {
            final BasicAttributes groupsAttributes = new BasicAttributes();
            final BasicAttribute groupsObjClass = new BasicAttribute("objectClass");
            groupsObjClass.add("top");
            groupsObjClass.add("organizationalUnit");
            groupsAttributes.put(groupsObjClass);
            groupsAttributes.put("ou", "groups");
            ctx.createSubcontext("ou=groups", groupsAttributes);
        } catch (NamingException e) {
            if (!((e instanceof NameAlreadyBoundException) || (e instanceof NoPermissionException))) {
                throw new LDAPSessionException(e);
            }
        }
    }

}
