package de.jalin.ldapadmin.ldap;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchResult;

import de.jalin.ldapadmin.beans.User;

public class UsersDAO {

    private final LDAPSession session;

    public UsersDAO(final LDAPSession session) {
        this.session = session;
    }

    public SortedMap<String, User> loadUsers() throws LDAPSessionException {
        final SortedMap<String, User> usersHash = new TreeMap<String, User>();
        final List<SearchResult> enumeration = session.search("ou=users");
        for (SearchResult result : enumeration) {
            final Attributes attribs = result.getAttributes();
            final User usr = new User();
            usr.setFirstname(session.getStringValue(attribs, "givenName"));
            usr.setLastname(session.getStringValue(attribs, "sn"));
            usr.setEmail(session.getStringValue(attribs, "mail"));
            usr.setLogin(session.getStringValue(attribs, "uid"));
            usr.setPhone(session.getStringValue(attribs, "telephoneNumber"));
            usr.setMobile(session.getStringValue(attribs, "mobile"));
            usr.setDisplayname(session.getStringValue(attribs, "displayName"));
            usr.setLogin(session.getStringValue(attribs, "uid"));
            usr.setDn(result.getNameInNamespace());
            // usr.setPassword(new String(session.getBytesValue(attribs, "userPassword")));
            usersHash.put(usr.getDn(), usr);
        }
        return usersHash;
    }

    public void create(final User usr) throws LDAPSessionException, RequiredAttributeException, AlreadyBoundException {
        assert usr != null;
        final String uid = usr.getLogin();
        if (!hasValue(uid)) {
            throw new RequiredAttributeException("uid");
        }
        assert uid != null;
        final BasicAttributes attributes = new BasicAttributes();
        final BasicAttribute objClass = new BasicAttribute("objectClass");
        objClass.add("top");
        objClass.add("person");
        objClass.add("organizationalPerson");
        objClass.add("inetOrgPerson");
        attributes.put(objClass);
        final String firstname = usr.getFirstname();
        if (hasValue(firstname)) {
            attributes.put("givenName", firstname);
        }
        final String lastname = usr.getLastname();
        if (!hasValue(lastname)) {
            throw new RequiredAttributeException("lastname");
        }
        attributes.put("sn", lastname);
        final BasicAttribute mail = new BasicAttribute("mail");
        final String email = usr.getEmail();
        if (hasValue(email)) {
            mail.add(email);
        }
        attributes.put(mail);
        attributes.put("uid", uid);
        attributes.put("cn", uid);
        final String telephone = usr.getPhone();
        if (hasValue(telephone)) {
            attributes.put("telephoneNumber", telephone);
        }
        final String mobile = usr.getMobile();
        if (hasValue(mobile)) {
            attributes.put("mobile", mobile);
        }
        final String comment = usr.getDisplayname();
        if (hasValue(comment)) {
            attributes.put("displayName", comment);
        } else {
            attributes.put("displayName", firstname == null ? lastname : firstname + " " + lastname);
        }
        final String password = usr.getPassword();
        if (!hasValue(password)) {
            throw new RequiredAttributeException("password");
        }
        attributes.put("userPassword", password);
        final String dn = session.createSubcontext("uid=${uid},ou=users".replace("${uid}", uid), attributes);
        usr.setDn(dn);
    }

    public User read(final String dn) throws LDAPSessionException {
        final Attributes attribs = session.getAttributes(dn.substring(0, dn.indexOf("ou=users") + 8));
        final User usr = new User();
        usr.setFirstname(session.getStringValue(attribs, "givenName"));
        usr.setLastname(session.getStringValue(attribs, "sn"));
        usr.setEmail(session.getStringValue(attribs, "mail"));
        usr.setLogin(session.getStringValue(attribs, "uid"));
        usr.setPhone(session.getStringValue(attribs, "telephoneNumber"));
        usr.setMobile(session.getStringValue(attribs, "mobile"));
        usr.setDisplayname(session.getStringValue(attribs, "displayName"));
        usr.setLogin(session.getStringValue(attribs, "uid"));
        usr.setDn(dn);
        return usr;
    }

    public void update(final User usr) throws LDAPSessionException {
        assert usr != null;
        final String uid = usr.getLogin();
        assert uid != null;
        final Attributes attribs = session.getAttributes(
            "uid=${uid},ou=users".replace("${uid}", uid));
        final List<ModificationItem> updates = new ArrayList<ModificationItem>();
        addStringAttrUpdate(updates, attribs, "displayName", usr.getDisplayname());
        final ArrayList<String> emailAdressList = new ArrayList<String>();
        final String email = usr.getEmail();
        if (hasValue(email)) {
        	emailAdressList.add(email);
        }
		addMultiValueAttrUpdate(updates, attribs, "mail", emailAdressList);
        addStringAttrUpdate(updates, attribs, "givenName", usr.getFirstname());
        addStringAttrUpdate(updates, attribs, "sn", usr.getLastname());
        addStringAttrUpdate(updates, attribs, "mobile", usr.getMobile());
        addStringAttrUpdate(updates, attribs, "telephoneNumber", usr.getPhone());
        addPasswordUpdate(updates, "userPassword", usr.getPassword());
        session.modifyAttributes("uid=${uid},ou=users".replace("${uid}", uid), updates.toArray(new ModificationItem[]{}));
    }

    public void delete(final User user) throws LDAPSessionException {
        assert user != null;
        session.unbind("uid=${id},ou=users", user.getLogin());
    }

    private boolean hasValue(final String param) {
        return param != null && !param.isEmpty();
    }

    private boolean hasValue(final List<String> param) {
        return param != null && !param.isEmpty();
    }

    private void addStringAttrUpdate(final List<ModificationItem> updates,
            final Attributes attribs, final String attributeName, final String newValue) throws LDAPSessionException {
        final String oldValue = session.getStringValue(attribs, attributeName);
        final BasicAttribute basicAttribute = new BasicAttribute(attributeName);
        if (hasValue(newValue)) {
            if (!newValue.equals(oldValue)) {
                basicAttribute.add(newValue);
                int ldapOp = DirContext.REPLACE_ATTRIBUTE;
                if (oldValue == null) {
                    ldapOp = DirContext.ADD_ATTRIBUTE;
                }
                final ModificationItem modificationItem = new ModificationItem(ldapOp, basicAttribute);
                updates.add(modificationItem);
            }
        } else {
            if (hasValue(oldValue)) {
                updates.add(new ModificationItem(DirContext.REMOVE_ATTRIBUTE, basicAttribute));
            }
        }
    }

    private void addMultiValueAttrUpdate(final List<ModificationItem> updates,
            final Attributes attribs, final String attributeName, final List<String> newValues) throws LDAPSessionException {
        final List<String> oldValues = session.getListOfValues(attribs, attributeName);
        final BasicAttribute basicAttribute = new BasicAttribute(attributeName);
        if (hasValue(newValues)) {
        	for (final String val : newValues) {
                basicAttribute.add(val);
        	}
            int ldapOp = DirContext.REPLACE_ATTRIBUTE;
            if (!hasValue(oldValues)) {
                ldapOp = DirContext.ADD_ATTRIBUTE;
            }
            updates.add(new ModificationItem(ldapOp, basicAttribute));
        } else {
            if (hasValue(oldValues)) {
                updates.add(new ModificationItem(DirContext.REMOVE_ATTRIBUTE, basicAttribute));
            }
        }
    }

    private void addPasswordUpdate(final List<ModificationItem> updates,
            final String attributeName, final String newValue) throws LDAPSessionException {
        if (hasValue(newValue)) {
            updates.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
                    new BasicAttribute(attributeName, newValue)));
        }
    }

}
